@@ -1,6 +1,6 @@
 #!/bin/bash
 
-# build script for manylinux builder on quay.io/pypa/manylinux2014_x86_64
+# build script for building manylinux wheels.
 
 
 set -e -u -x
@@ -9,16 +9,15 @@ set -e -u -x
 # add builder user and su to build to avoid creating root-owned files in host side.
 groupadd --gid $GROUP_ID builder
 useradd --no-create-home --comment 'Builder' --uid $USER_ID --gid $GROUP_ID builder
+git config --global --add safe.directory /io
+
+# apt install -y libssl-dev
+# apt install -y zlib1g-dev
 
 export PYTHONDONTWRITEBYTECODE=1
 
-# Install a system package required by our library
-yum install -y zlib-devel
-yum install -y openssl-devel
-
 LIBC=$(getconf GNU_LIBC_VERSION | tr -d ' ')
 
-
 BUILDDIR="/io/build/${LIBC}"
 
 mkdir -p $BUILDDIR
@@ -37,16 +36,17 @@ su builder -c "cmake --build . --config Release"
 popd
 
 # So that loader and auditwheel can find libssh2
-LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${BUILDDIR}/src
+export LD_LIBRARY_PATH=${BUILDDIR}/src
 
-declare -a PYS=(cp36-cp36m cp37-cp37m cp38-cp38 cp39-cp39)
+
+declare -a PYTHONS=(cp37-cp37m cp38-cp38 cp39-cp39 cp310-cp310 cp311-cp311)
 
 pushd /io
 
-for PY in ${PYS[@]} ; do
+for PY in ${PYTHONS[@]} ; do
     PYBIN=/opt/python/${PY}/bin
-    "${PYBIN}/pip" install -U pip setuptools auditwheel
     "${PYBIN}/pip" install -r requirements_dev.txt
+    rm -f ssh2/*.c
     su builder -c "\"${PYBIN}/python\" setup.py bdist_wheel --plat-name $PLAT"
     whl=$(echo dist/*-${PY}-${PLAT}.whl)
     su builder -c "\"${PYBIN}/python\" -m auditwheel repair --plat \"$PLAT\" --wheel-dir /io/wheelhouse/ \"$whl\""
