@@ -2254,9 +2254,9 @@ void
 _libssh2_dh_init(_libssh2_dh_ctx *dhctx)
 {
     /* Random from client */
-    dhctx->bn = NULL;
     dhctx->dh_handle = NULL;
     dhctx->dh_params = NULL;
+    dhctx->dh_privbn = NULL;
 }
 
 void
@@ -2272,9 +2272,9 @@ _libssh2_dh_dtor(_libssh2_dh_ctx *dhctx)
         free(dhctx->dh_params);
         dhctx->dh_params = NULL;
     }
-    if(dhctx->bn) {
-        _libssh2_wincng_bignum_free(dhctx->bn);
-        dhctx->bn = NULL;
+    if(dhctx->dh_privbn) {
+        _libssh2_wincng_bignum_free(dhctx->dh_privbn);
+        dhctx->dh_privbn = NULL;
     }
 }
 
@@ -2419,25 +2419,26 @@ _libssh2_dh_key_pair(_libssh2_dh_ctx *dhctx, _libssh2_bn *public,
 
         if(dh_key_blob->dwMagic == BCRYPT_DH_PRIVATE_MAGIC) {
             /* BCRYPT_DH_PRIVATE_BLOB additionally contains the Private data */
-            dhctx->bn = _libssh2_wincng_bignum_init();
-            if(!dhctx->bn) {
+            dhctx->dh_privbn = _libssh2_wincng_bignum_init();
+            if(!dhctx->dh_privbn) {
                 _libssh2_wincng_safe_free(blob, key_length_bytes);
                 return -1;
             }
-            if(_libssh2_wincng_bignum_resize(dhctx->bn, dh_key_blob->cbKey)) {
+            if(_libssh2_wincng_bignum_resize(dhctx->dh_privbn,
+                                             dh_key_blob->cbKey)) {
                 _libssh2_wincng_safe_free(blob, key_length_bytes);
                 return -1;
             }
 
             /* Copy the private key data into the dhctx bignum data buffer */
-            memcpy(dhctx->bn->bignum,
+            memcpy(dhctx->dh_privbn->bignum,
                    blob + sizeof(*dh_key_blob) + 3 * dh_key_blob->cbKey,
                    dh_key_blob->cbKey);
 
             /* Make sure the private key is an odd number, because only
              * odd primes can be used with the RSA-based fallback while
              * DH itself does not seem to care about it being odd or not. */
-            if(!(dhctx->bn->bignum[dhctx->bn->length-1] % 2)) {
+            if(!(dhctx->dh_privbn->bignum[dhctx->dh_privbn->length-1] % 2)) {
                 _libssh2_wincng_safe_free(blob, key_length_bytes);
                 /* discard everything first, then try again */
                 _libssh2_dh_dtor(dhctx);
@@ -2450,12 +2451,12 @@ _libssh2_dh_key_pair(_libssh2_dh_ctx *dhctx, _libssh2_bn *public,
     }
 
     /* Generate x and e */
-    dhctx->bn = _libssh2_wincng_bignum_init();
-    if(!dhctx->bn)
+    dhctx->dh_privbn = _libssh2_wincng_bignum_init();
+    if(!dhctx->dh_privbn)
         return -1;
-    if(_libssh2_wincng_bignum_rand(dhctx->bn, group_order * 8 - 1, 0, -1))
+    if(_libssh2_wincng_bignum_rand(dhctx->dh_privbn, (group_order*8)-1, 0, -1))
         return -1;
-    if(_libssh2_wincng_bignum_mod_exp(public, g, dhctx->bn, p))
+    if(_libssh2_wincng_bignum_mod_exp(public, g, dhctx->dh_privbn, p))
         return -1;
 
     return 0;
@@ -2588,7 +2589,7 @@ out:
 
 fb:
     /* Compute the shared secret */
-    return _libssh2_wincng_bignum_mod_exp(secret, f, dhctx->bn, p);
+    return _libssh2_wincng_bignum_mod_exp(secret, f, dhctx->dh_privbn, p);
 }
 
 /* _libssh2_supported_key_sign_algorithms
